package com.zfire.jiasm.syncdata.upload.order;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.zfire.jiasm.syncdata.base.BaseService;
import com.zfire.jiasm.syncdata.constant.OrderPGStatusEnum;
import com.zfire.jiasm.syncdata.constant.OrderStatusEnum;
import com.zfire.jiasm.syncdata.dao.mapper.GDOrderBaseMapper;
import com.zfire.jiasm.syncdata.dao.mapper.OJTGUploadTaskListMapper;
import com.zfire.jiasm.syncdata.parameter.InstallUpdatePar;
import com.zfire.jiasm.syncdata.parameter.OrderPar;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.service.*;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * 生成需要上传的数据
 * 数据准备类型定义表提取数据 out_jiasm_to_gree_upload_task_type
 * 根据类型定义表的任务类型 从数据上传任务表提取对应的任务类型数据
 * 生成相应的upload数据插入表中
 * 修改上传任务表的状态
 */

@Slf4j
@Component
public class ProduceDataUploadList {

    @Resource
    ProduceDataUploadList produceDataUploadList;
    @Resource
    OutJiasmToGreeUploadTaskTypeService outJiasmToGreeUploadTaskTypeService;
    @Resource
    OutJiasmToGreeUploadTaskListService outJiasmToGreeUploadTaskListService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    GDOrderBaseMapper gdOrderBaseMapper;
    @Resource
    BaseService baseService;
    @Resource
    BrandLbsRelateService brandLbsRelateService;
    @Resource
    AreaService areaService;
    @Resource
    StreetService streetService;
    @Resource
    OJTGUploadTaskListMapper ojtgUploadTaskListMapper;
    @Resource
    ItfTblAzAssignLcLsUploadService itfTblAzAssignLcLsUploadService;
    @Resource
    OrderInstallDetailService orderInstallDetailService;




    @Scheduled(fixedDelay = 1 * 20 * 1000)
    private void produceList(){
        /*if (1==1){
            return;
        }*/

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        // 提取品牌信息
        Brand brand = baseService.getBrand();
        if (brand == null) {
            return;
        }

        log.info("开始生成需要上传的订单数据: {}",sdf.format(new Date()));

        // 数据准备类型定义表中提取所有数据
        List<OutJiasmToGreeUploadTaskType> ojtutts = outJiasmToGreeUploadTaskTypeService.lambdaQuery()
                .orderByAsc(OutJiasmToGreeUploadTaskType::getId).list();
        try{
            for (OutJiasmToGreeUploadTaskType item:ojtutts){
                // 任务类型
                int uploadTaskType = item.getUploadTaskType();

                // 数据上传任务表提取对应的任务类型数据
                List<OutJiasmToGreeUploadTaskList> ojtutls = outJiasmToGreeUploadTaskListService.lambdaQuery()
                        .eq(OutJiasmToGreeUploadTaskList::getSynStatus,0)
                        .eq(OutJiasmToGreeUploadTaskList::getUploadTaskType,uploadTaskType)
                        .orderByAsc(OutJiasmToGreeUploadTaskList::getSynReqId).list();
                for (OutJiasmToGreeUploadTaskList ojtutl:ojtutls){
                    // 安装工单数据生成
                    // 同步任务表中的主键,安装工单号,源表主键
                    // TL_PrePareSynDataby_fa_order_install_test

                    // 安装工单 基础表数据同步
                    if (ojtutl.getTableName().equals("order_base") && ojtutl.getOrderType().equals("INSTALL")){
                        if (orderBaseInstall(brand, ojtutl)) {
                            continue;
                        }
                    }
                }
            }

            log.info("生成需要上传的订单数据结束: {}",sdf.format(new Date()));

        } catch (Exception ex) {
            ex.printStackTrace();
            log.info("开始预处理生成需要上传的数据出错");
        }
    }

    private boolean orderBaseInstall(Brand brand, OutJiasmToGreeUploadTaskList ojtutl) {
        OrderBase orderBase = orderBaseService.lambdaQuery()
                .eq(OrderBase::getId, ojtutl.getId()).one();

        if (orderBase !=null ){
            // 若工单被删除不再同步 不写生成数据

            // 接收派工网点为空上传忽略
            if (checkPare(ojtutl, orderBase)) {
                return true;
            }


            // 安装工单产品信息,操作日志,完工明细,完工明细附件 以主表的pgguid统一修改
            // 生成待同步的工单主表数据
            ItfTblAzAssignLcLsUpload lcls = getItfTblAzAssignLcLsUpload(brand, ojtutl);

            // 差辅材订单表逻辑 生成 空调安装费用清单上传记录数据

            // 生成采集数据及家用，商用完工明细数据
            // 工单状态order_status 6, 7, 9, 10 且为非预派工订单
            if ((orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()) ||
                    orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode()) ||
                    orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                    orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode())) &&
                    orderBase.getIsReservation().equals(0)){
                // 提取采集的数据
                List<OrderInstallDetail> orderInstallDetails = orderInstallDetailService.lambdaQuery()
                        .eq(OrderInstallDetail::getOrderBaseId,orderBase.getId()).list();
                if (orderInstallDetails !=null && orderInstallDetails.size()>0){
                    for (OrderInstallDetail detail :orderInstallDetails){
                        // 状态:3=临时保存;2=作废,1=已采集,0=未采集
                        if (detail.getStatus().equals(0) || detail.getStatus().equals(3)){
                            continue;
                        }
                        // 修改 pgwcmxid
                        OrderInstallDetail updetail = new OrderInstallDetail();
                        updetail.setId(detail.getId());
                        updetail.setPgwcmxid(UUID.randomUUID().toString());

                        // OrderInstallDetail 表的pgguid 统一改成工单基础表的 pgguid

                        // 为空时为 家用空调
                        if (StringUtils.isEmpty(orderBase.getMainName()) ||
                                orderBase.getMainName().equals("家用空调") ||
                                orderBase.getMainName().equals("家用空气能热水器") ||
                                orderBase.getMainName().equals("驻车空调")){
                            // 家用空调 数据生成

                        } else if (orderBase.getMainName().equals("商用空调")){
                            // 商用空调 数据生成

                        } else {
                            // 其他 数据生成

                        }

                        if (orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode()) ||
                                orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode())){
                            // 处理附件采集数据上传(需要增加附件上传总部标识判断!!!)

                        }

                        // 处理条码明细


                    }

                }
            }

            // 插入生成的新数据，修改任务表任务状态
            produceDataUploadList.addAndUpdate(ojtutl, lcls);
        }
        return false;
    }

    @NotNull
    private ItfTblAzAssignLcLsUpload getItfTblAzAssignLcLsUpload(Brand brand, OutJiasmToGreeUploadTaskList ojtutl) {
        OrderPar orderPar = new OrderPar();
        orderPar.setBrandId(brand.getBrandId());
        orderPar.setOrderNo(ojtutl.getId());
        ItfTblAzAssignLcLsUpload lcls = gdOrderBaseMapper.getAZOrderBase(orderPar);
        lcls.setUploadTaskSynReqId(ojtutl.getSynReqId());
        // 省市等信息单独处理
        setssxz(lcls);
        return lcls;
    }

    @Transactional(rollbackFor = Exception.class)
    public void addAndUpdate(OutJiasmToGreeUploadTaskList ojtutl, ItfTblAzAssignLcLsUpload lcls) {

        // 插入工单主表数据
        itfTblAzAssignLcLsUploadService.save(lcls);

        // 修改数据上传任务表中的状态
        updateUploadTaskList(ojtutl);
    }

    private void updateUploadTaskList(OutJiasmToGreeUploadTaskList ojtutl) {
        OutJiasmToGreeUploadTaskList upoutgultl = new OutJiasmToGreeUploadTaskList();
        upoutgultl.setSynReqId(ojtutl.getSynReqId());
        upoutgultl.setSynStatus(1);
        upoutgultl.setSynTime(new Date());
        outJiasmToGreeUploadTaskListService.updateById(upoutgultl);
        // 批量修改
        InstallUpdatePar request = new InstallUpdatePar();
        request.setOrderNo(ojtutl.getId());
        request.setUploadTaskType(22);
        request.setSynReqId(ojtutl.getSynReqId());
        ojtgUploadTaskListMapper.installUpdateuploadTaskList(request);
    }

    private boolean checkPare(OutJiasmToGreeUploadTaskList ojtutl, OrderBase orderBase) {
        if (StringUtils.isEmpty(orderBase.getReceWebsitId()) || orderBase.getReceWebsitId().equals("0")){
            stopUpload(ojtutl, "接收派工网点为空上传忽略");
            return true;
        }
        if (StringUtils.isEmpty(orderBase.getPgguid()) || orderBase.getPgguid().equals("0")){
            stopUpload(ojtutl, "pgguid异常上传忽略");
            return true;
        }
        if (StringUtils.isEmpty(orderBase.getPgid()) || orderBase.getPgid().equals("0")){
            stopUpload(ojtutl, "pgid异常上传忽略");
            return true;
        }
        return false;
    }

    private void setssxz(ItfTblAzAssignLcLsUpload lcls){
        List<BrandLbsRelate> br;
        // 街道处理
        if (StringUtils.isNotEmpty(lcls.getXzhenid())){
            br = brandLbsRelateService.lambdaQuery()
                    .eq(BrandLbsRelate::getLbsStreetId, lcls.getXzhenid())
                    .isNotNull(BrandLbsRelate::getStreetId).list();
            if (br !=null && br.size()>0){
                lcls.setXzhenid(br.get(0).getStreetId());

                Street street = streetService.lambdaQuery()
                        .eq(Street::getProvince, lcls.getSfenid())
                        .eq(Street::getCity, lcls.getCshiid())
                        .eq(Street::getArea, lcls.getXianid())
                        .eq(Street::getStreet, lcls.getXzhenid()).one();
                if (street !=null){
                    lcls.setXzhen(street.getStreet());
                    lcls.setSfenid(Long.valueOf(street.getProvinceId()));
                    lcls.setCshiid(Long.valueOf(street.getCityId()));
                    lcls.setXianid(Long.valueOf(street.getAreaId()));

                    lcls.setSfen(street.getProvince());
                    lcls.setCshi(street.getCity());
                    lcls.setXian(street.getArea());
                    return;
                }
            }
        }

        //省
        br = brandLbsRelateService.lambdaQuery()
                .eq(BrandLbsRelate::getLbsProvinceId, lcls.getSfenid())
                .isNotNull(BrandLbsRelate::getProvinceId).list();
        if (br !=null && br.size()>0){
            lcls.setSfenid(Long.valueOf(br.get(0).getProvinceId()));
        }
        //市
        br = brandLbsRelateService.lambdaQuery()
                .eq(BrandLbsRelate::getLbsCityId, lcls.getCshiid())
                .isNotNull(BrandLbsRelate::getCityId).list();
        if (br !=null && br.size()>0){
            lcls.setCshiid(Long.valueOf(br.get(0).getCityId()));
        }
        //区
        br = brandLbsRelateService.lambdaQuery()
                .eq(BrandLbsRelate::getLbsDistrictId, lcls.getXianid())
                .isNotNull(BrandLbsRelate::getAreaId).list();
        if (br !=null && br.size()>0){
            lcls.setXianid(Long.valueOf(br.get(0).getAreaId()));
        }

        // 总部区必选
        Area area = areaService.lambdaQuery()
                .eq(Area::getAreaId, lcls.getXianid()).one();
        Street st = new Street();
        if (area !=null){
            lcls.setSfen(area.getProvince());
            lcls.setCshi(area.getCity());
            lcls.setXian(area.getArea());
        }
    }




    private void stopUpload(@NotNull OutJiasmToGreeUploadTaskList ojtutl, String msg) {
        OutJiasmToGreeUploadTaskList up = new OutJiasmToGreeUploadTaskList();
        up.setSynReqId(ojtutl.getSynReqId());
        up.setSynStatus(99);
        up.setSynTime(new Date());
        up.setSynErrMsg(msg);
        outJiasmToGreeUploadTaskListService.updateById(up);
    }

}
